\name{createMPS.Rd}
\alias{write_MPSILP}
\alias{write_MPSILPlist}
\alias{write_MPSMIQP}
\alias{create_ILP_list}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Functions to create lists of (mixed) integer quadratic or linear problems related to orthogonal arrays
}
\description{
create_ILP_list creates a list of problems in Mosek formatting. write_MPSILP saves a single integer linear problem in MPS format, 
write_MPSILP_list saves a list of integer linear problems as separate MPS files that are accompanied by a table of content txt file. 
All functions work, if neither Mosek nor Gurobi is available.
}
\usage{
create_ILP_list(nruns, nlevels, resolution=3, distinct=TRUE, 
              search.orders=TRUE, start=NULL, forced=NULL, orders=NULL)
write_MPSILP(file, qco=NULL, qcoinfo=NULL, 
              cvec=NULL, bvec=NULL, Amat=NULL, bsense=NULL, boundmat=NULL, 
              name = "EnforceResolution", commentline="* no comment")
write_MPSILPlist(prefix, qcolist, toc=TRUE)
write_MPSMIQP(prefix, nruns, nlevels, resolution=3, distinct=TRUE, start=NULL, 
              forced=NULL, name="ImproveAR", commentline="* quadratic problem")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{nruns}{
positive integer; number of runs
}
  \item{nlevels}{
vector of integers (>=2); numbers of factor levels
}
  \item{resolution}{
positive integer; the minimum resolution requested
}
  \item{distinct}{
logical; if TRUE (default), restricts counting vector to 0/1 entries, which means that the resulting array is requested to have distinct rows; otherwise, duplicate rows are permitted, i.e. the counting vector can have arbitrary non-negative integers. Designs with distinct runs are usually better; in addition, binary variables are easier to handle by the optimization algorithm. Nevertheless, there are occasions where a better array is found faster with option \code{distinct=FALSE}, even if it has distinct rows.
}
  \item{search.orders}{
logical (default TRUE); determines whether a list of arrays for different level orderings is produced or a single array is output only
}
  \item{start}{
a starting value for the algorithm: can be a array matrix with entries 1 to number of levels for each column, or a counting vector for the full factorial in lexicographic order; if specified, \code{start} must specify an array with the appropriate number of rows and columns, the requested resolution and, if \code{distinct = TRUE}, also contain distinct rows (matrix) or 0/1 elements only. \code{start} cannot be combined with \code{search.orders=TRUE}.
}
  \item{forced}{
for \code{resolution > 1} only;\cr
runs to force into the solution design; can be given as an array matrix with the appropriate number of columns and less than \code{nruns} rows or a counting vector for the full factorial in lexicographic order with sum smaller than \code{nruns}; if \code{distinct=TRUE}, \code{forced} must have distinct rows (matrix) or 0/1 elements only.
}
  \item{orders}{
a list of level orderings to be considered; if \code{orders} is not specified but \code{search.orders=TRUE}, all distinct level orderings 
}
  \item{file}{
character string that provides the path to the save file
}
  \item{qco}{
an object of class qco that can be used as input for function \code{Rmosek::mosek}
}
  \item{qcoinfo}{
contains information on the \code{qco} object (keeping that information in \code{qco} itself hinders execution of \code{Rmosek::mosek})
}
  \item{cvec}{
coefficients of linear objective function
}
  \item{bvec}{
RHS of the constraints
}
  \item{Amat}{
matrix of coefficients for the LHS of the constraints
}
  \item{bsense}{
directions of constraints (character vector with elements \code{"E"} for equal, \code{"G"} for greater, \code{"L"} for less or \code{"N"} for not restricted.
}
  \item{boundmat}{
matrix of lower (first row) and upper (second row) bounds for the variables
}
  \item{name}{
name to be written into the name field of the mps file
}
  \item{commentline}{
comment to be written directly underneath the name (comments have to start with a *)
}
  \item{prefix}{
file name prefix to be supplemented with a number and the .mps suffix later. In addition, a further file with a different suffix may be created (see details).
}
  \item{qcolist}{
list of ILP objects in Mosek notation (as e.g. produced by function \code{create_ILP_list})
}
  \item{toc}{
logical (default TRUE) that indicates whether a table of content should be produced (txt file)
  }
}
\details{
\code{create_ILP_list} creates a list with one or more integer linear problems for creating designs of at least the specified resolution. The problems are in mosek formatting.

\code{write_MPSILP} writes a single problem to an MPS file, \code{write_MPSILPlist} writes a list of several problems to several MPS files. If not suppressed, a toc file is also created (suffix \code{_toc.txt}), which contains the number of runs, the target resolution and the elements of \code{nlevels} for each MPS file.

\code{write_MPSMIQP} creates and writes a problem for improving the number of shortest words, in a form that corresponds to general MPS format, without extensions by Mosek or Gurobi. The problem has a quadratic objective like in Groemping and Fontana (2019) Eq.(2Q) (instead of Eq.(2L), which is solved by Gurobi and Mosek). The start array should be available from a previous linear optimization (though this is not enforced). It cannot be stored in the MPS file; instead, for a non-NULL start array, a separate file (suffix \code{.start}) is created, and users have to figure out how they can make their solver use that start solution. If an admissible start array is provided, \code{write_MPSMIQP} initially prints the GWLP of that start array. Otherwise, it warns of inadmissibility.

Note that it can take a long time to write the problems, if the problem has many variables (the number of variables is \code{prod(nlevels)}). 

Note that the functions do not do any problem solving!
}
\value{
Only functions \code{write_MPSILPlist} and \code{write_MPSMIQP} potentially return a value: 
\code{write_MPSILPlist} returns the matrix that is also written to the table of contents file, \code{write_MPSMIQP} returns the start vector (if \code{start} is not NULL). 
}
\references{
Fontana, R. (2017). Generalized Minimum Aberration mixed-level orthogonal arrays: a general approach based on sequential integer quadratically constrained quadratic programming. \emph{Communications in Statistics – Theory Methods} \bold{46}, 4275-4284.

Groemping, U. and Fontana, R. (2019). An Algorithm for Generating Good Mixed Level Factorial Designs. \emph{Computational Statistics & Data Analysis} \bold{137}, 101-114. doi:10.1016/j.csda.2019.01.020.

Gurobi Optimization Inc. (2017). Gurobi Optimizer Reference Manual. \url{http://www.gurobi.com/documentation/}.

Mosek ApS (2017a). MOSEK version w.x.y.z documentation. Accessible at: \url{https://www.mosek.com/documentation/}. This package has been developed using version 8.1.0.23 (accessed August 29 2017).

Mosek ApS (2017b). MOSEK Rmosek Package 8.1.y.z. \url{http://docs.mosek.com/8.1/rmosek/index.html}.
\emph{!!! In normal R speak, this is the documentation of the Rmosek package version 8.0.69 (or whatever comes next), when applied on top of the Mosek version 8.1.y.z (this package has been devoloped with Mosek version 8.1.0.23 and will likely not work for Mosek versions before 8.1). !!! (accessed August 29 2017)}

}

\note{
The functions are not meant for situations, for which a full factorial design would be huge. Even though the functions do not solve anything, MPS files will be very large and writing them will be quite slow for such cases.
}

\author{
Ulrike Groemping
}

\seealso{
See also \code{\link{mosek_MIPsearch}} and \code{\link{gurobi_MIPsearch}} for searching over \code{nlevels} orderings,
\code{\link{mosek_MIPcontinue}} and \code{\link{gurobi_MIPcontinue}} for continuing an uncompleted optimization,
\code{\link[DoE.base]{show.oas}} from package \pkg{DoE.base} for catalogued orthogonal arrays,
and \code{\link[DoE.base]{oa_feasible}} from package \pkg{DoE.base} for checking feasibility of requested array strength (resolution - 1) for combinations of \code{nruns} and \code{nlevels}.
}
\examples{
\dontrun{
## can also be run with gurobi_MIParray instead of mosek_MIParray
## there are of course better ways to obtain good arrays for these parameters
## (e.g. function FrF2 from package FrF2)
problemlist <- create_ILP_list(16, c(2,2,4,8), resolution=2, kmax=4)
feld
names(attributes(feld))
attr(feld, "MIPinfo")$info

## using a start value
start <- DoE.base::L16.2.8.8.1[,1:5]
feld <- mosek_MIParray(16, rep(2,5), resolution=4, start=start)

## counting vector representation of the start value could also be used
DoE.MIParray:::dToCount(start-1)
   ## "-1", because the function requires values starting with 0
   ## 32 elements for the full factorial in lexicographic order, 16 ones for the runs

## extending an existing array
force <- matrix(as.numeric(as.matrix(DoE.base::undesign(DoE.base::oa.design(L8.2.7)))), nrow=8)
feld <- mosek_MIParray(16, rep(2,7), resolution=3, kmax=4, forced=force)
attr(feld, "MIPinfo")$info
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ array }% use one of  RShowDoc("KEYWORDS")
\keyword{ design }% __ONLY ONE__ keyword per line
